'''
The rows of levels in the .azr file are converted to list of strings. These
indices make it more convenient to access the desired parameter.
'''
J_INDEX = 0
PI_INDEX = 1
ENERGY_INDEX = 2
ENERGY_FIXED_INDEX = 3
CHANNEL_INDEX = 5
WIDTH_INDEX = 11
WIDTH_FIXED_INDEX = 10
SEPARATION_ENERGY_INDEX = 21
CHANNEL_RADIUS_INDEX = 27
OUTPUT_DIR_INDEX = 2
DATA_FILEPATH_INDEX = 11
LEVEL_INCLUDE_INDEX = 9


'''
Data-related constants.
'''
INCLUDE_INDEX = 0
IN_CHANNEL_INDEX = 1
OUT_CHANNEL_INDEX = 2
NORM_FACTOR_INDEX = 8
VARY_NORM_FACTOR_INDEX = 9
FILEPATH_INDEX = 11
